<?php
/**
 * API handler class
 * Contains all functions to interact with the ParcelBroker API
 */
trait ParcelbrokerApiTrait
{
	// Meta key for the order tracking
	protected $parcelbrokerwc_tracking_key = '_pb_tracking_reference';
	// API url
	protected $parcelbrokerwc_api_url = 'https://parcelbroker.co.uk/api';
	// Editable settings used in conjunction with the API
	protected $parcelbrokerwc_settings = [
		'api_key' => 'parcelbrokerwc-api_key',
		'api_token' => 'parcelbrokerwc-api_token',
		'powered_by' => 'parcelbrokerwc-powered_by',
		'dismiss_notice' => 'parcelbrokerwc-dismiss_notice',
		'use_shipping' => 'parcelbrokerwc-use_shipping',
		'use_shipping_dimensions' => 'parcelbrokerwc-use_shipping_dimensions',
		'use_test_shipping' => 'parcelbrokerwc-use_test_shipping',
		'collection_name' => 'parcelbrokerwc-collection_name',
		'collection_company' => 'parcelbrokerwc-collection_company',
		'collection_address_1' => 'parcelbrokerwc-collection_address_1',
		'collection_address_2' => 'parcelbrokerwc-collection_address_2',
		'collection_town' => 'parcelbrokerwc-collection_town',
		'collection_state' => 'parcelbrokerwc-collection_state',
		'collection_zip' => 'parcelbrokerwc-collection_zip',
		'collection_country' => 'parcelbrokerwc-collection_country',
		'collection_phone' => 'parcelbrokerwc-collection_phone',
		'collection_email' => 'parcelbrokerwc-collection_email'
	];

	// List of rate ids with surcharge requirements
	protected $parcelbrokerwc_surcharge_list = [72,80,82,83,87,89,92,93,95,102,103,105,174,175,176];
	protected $parcelbrokerwc_dropoff = [85,202, 41, 65, 66, 67, 68, 125];
	protected $parcelbrokerwc_samedaycollection = [114, 116, 117, 124, 125, 109, 110, 11, 112, 113, 51, 53, 54];

	/**
	 * Api caller
	 * @param  string $body   XML body for the api call
	 * @param  string $method API method to call
	 * @return JSON         json encoded response from the API clal
	 */
	private function apiCall($body, $method = 'tracking')
	{
		$token = sha1(get_option($this->parcelbrokerwc_settings['api_token']) . '_publickey');
		$api_key = get_option($this->parcelbrokerwc_settings['api_key']);
		$uniqueid = wp_generate_password(100, false, false);
		$xml = "<{$method}><header><apikey>{$api_key}</apikey><key>publickey</key><token>{$token}</token><uniqueid>{$uniqueid}</uniqueid></header>{$body}</{$method}>";
		$url = $this->parcelbrokerwc_api_url;
		$response = wp_safe_remote_post(
		    $url,
		    array(
		        'method' => 'POST',
		        'timeout' => 45,
		        'redirection' => 5,
		        'httpversion' => '1.0',
		        'headers' => array(
		            'Content-Type' => 'application/x-www-form-urlencoded'
		        ),
		        'body' => ['xmldata' => $xml],
		        'sslverify' => false
		    )
		);
		// Format response into JSON format
		return json_decode(json_encode(simplexml_load_string(utf8_encode($response['body']))), TRUE);
	}

	/**
	 * Tracking API call
	 * @param  string $tracking_ref Order tracking reference
	 * @return JSON               response from API call
	 */
	public function getApiTracking($tracking_ref)
	{
		$body = "<ordernumber>{$tracking_ref}</ordernumber>";
		return $this->apiCall($body);
	}

	/**
	 * Carrier pricing call
	 * @param  Array $collection_address Array of collection address details
	 * @param  Array $delivery_address   Array of delivery address details
	 * @param  Array $items              Array of parcels
	 * @param  string $type              Package type
	 * @return JSON               response from API call
	 */
	public function getApiShipping($collection_address, $delivery_address, $items, $type = 'PARCEL')
	{
		$body = "<shipment><type>" . strtoupper($type) . "</type><collection><town>{$collection_address['town']}</town><postcode>{$collection_address['postcode']}</postcode><countrycode>{$collection_address['countrycode']}</countrycode></collection><destination><town>{$delivery_address['town']}</town><postcode>{$delivery_address['postcode']}</postcode><countrycode>{$delivery_address['countrycode']}</countrycode></destination><parcels>";

		foreach($items as $item) {
			$body .= "<parcel><weight>{$item['weight']}</weight><length>{$item['length']}</length><width>{$item['width']}</width><height>{$item['height']}</height></parcel>";
		}

		$body .= "</parcels></shipment>";

		// Get API response
		$response = $this->apiCall($body, 'lookup');

		if( $response ) {
			$json_data = json_decode(file_get_contents(__DIR__ . '/Admin/rate_surcharge_data.json'));
			// Loop through services and assign surcharge rates to appropriate rates
			foreach( $response['services']['service'] as $k => $service ) {
				if( !empty($json_data->{$service['rate_id']}) ) {
					$response['services']['service'][$k]['surcharges'] = $json_data->{$service['rate_id']};
				}
			}
		}

		return $response;
	}

	/**
	 * Booking request call
	 * @param  String $shipping       String containing rate id and hash
	 * @param  Object $theorder       The order object
	 * @param  Array $collection      Array of collection addres information
	 * @param  Array $delivery        Array of delivery address information
	 * @param  Array $parcels         Array of parcels
	 * @param  Array $customs         Array of customs information
	 * @param  String $packaging      Packaging type
	 * @param  Date $collection_date  Collection date
	 * @param  String $collection_type Collection Date requirement
	 * @return JSON               response from API call, XML request and parcel list
	 */
	public function sendApiCollection($shipping, $theorder, $collection, $delivery, $parcels, $shipping_contents, $customs, $packaging, $collection_date, $collection_type)
	{
		// Test mode flag
		$test = '';
		if(get_option($this->parcelbrokerwc_settings['use_test_shipping'])) {
			$test = "<test>true</test>";
		}
		$ordernumber = trim(str_replace('#', '', $theorder->get_order_number()));
		list($rate_id, $hash) = explode(':', $shipping);
		$parcel_list = [];

		$locales = WC()->countries->get_country_locale();
		$delivery_locale = $locales[$delivery['country_code']];
		$collection_locale = $locales[$collection['country_code']];

		// Set postcode to the town if locale information doesnt require postcode and the postcode is not set
		if( !empty($delivery_locale) ) {
			if(!empty($delivery_locale['postcode']) && isset($delivery_locale['postcode']['required']) && !$delivery_locale['postcode']['required']) {
				$delivery['postcode'] = $delivery['town'];
			}
		}

		// Set postcode to the town if locale information doesnt require postcode and the postcode is not set
		if( !empty($collection_locale) ) {
			if(!empty($collection_locale['postcode']) && isset($collection_locale['postcode']['required']) && !$collection_locale['postcode']['required']) {
				$collection['postcode'] = $delivery['town'];
			}
		}

		$shipping_contents = trim($shipping_contents);

		$xml =
		"<shipment>
	        {$test}
	        <rate_id>{$rate_id}</rate_id>
	        <hash>{$hash}</hash>
	        <customref>Order# {$ordernumber}</customref>
	        <contents>{$shipping_contents}</contents>
	        <estvalue>{$theorder->get_total()}</estvalue>
	        <liability>false</liability>
	        <type>PARCEL</type>
	        <collection>
	            <name>{$collection['name']}</name>
	            <company>{$collection['company']}</company>
	            <phone>{$collection['phone']}</phone>
	            <email>{$collection['email']}</email>
	            <line1>{$collection['address_1']}</line1>
	            <line2>{$collection['address_2']}</line2>
	            <line3></line3>
	            <town>{$collection['town']}</town>
	            <county>{$collection['state']}</county>
	            <postcode>{$collection['postcode']}</postcode>
	            <countrycode>{$collection['country_code']}</countrycode>
	        </collection>
	        <destination>
	            <name>{$delivery['name']}</name>
	            <company>{$delivery['company']}</company>
	            <phone>{$delivery['phone']}</phone>
	            <email>{$delivery['email']}</email>
	            <line1>{$delivery['address_1']}</line1>
	            <line2>{$delivery['address_2']}</line2>
	            <line3></line3>
	            <town>{$delivery['town']}</town>
	            <county>{$delivery['state']}</county>
	            <postcode>{$delivery['postcode']}</postcode>
	            <countrycode>{$delivery['country_code']}</countrycode>
	        </destination>
	        <parcels>";
	        foreach ($parcels as $parcel) {
	        	for($i = 0; $i < $parcel['qty']; $i++) {
	        		$parcel_list[] = $parcel;
					$xml .=
		        	"<parcel>
		                <weight>{$parcel['weight']}</weight>
		                <length>{$parcel['length']}</length>
		                <width>{$parcel['width']}</width>
		                <height>{$parcel['height']}</height>
		                <packaging>{$packaging}</packaging>
		            </parcel>";
	        	}
	        }
	        switch($collection_type) {
	        	case "additional-sameday" :
	        	case "sameday" :
	        		$pickup = "<today>true</today>";
	        	break;
	        	case "dropoff" :
	        		$pickup = "<dropoff>true</dropoff>";
	        	break;
	        	default:
	        		$pickup = "<date>{$collection_date}</date>";
	        	break;
	        }
			$xml .=
	        "</parcels>
	        <pickup>
				{$pickup}
			</pickup>";
			if (!empty($customs['use']) && $customs['use'] != 'false' ) {

				$xml .= "
		        <customs>
		            <type>{$customs['type']}</type>
		            <sendervat>{$customs['sender_vat']}</sendervat>
		            <receivervat></receivervat>
		            <com>{$customs['com']}</com>
		            <reason>" . strtoupper($customs['reason']) . "</reason>
		            <contents>";

				    foreach( $theorder->get_items() as $item ) {
				    	$product = $item->get_product();
				    	$product_data = $item->get_data();
				    	if ($product->is_virtual() ) {
				    		continue;
				    	}

				    	$tax = $product_data['total_tax'];
				    	if($tax > 0) {
				    		$tax /= $item['qty'];
				    	}

						$xml .=
				    	"<content>
							<description>{$product->get_title()}</description>
							<quantity>{$item['qty']}</quantity>
							<value>" . number_format((float)($product->get_price() + $tax), 2) . "</value>
				    	</content>";
				    }

				$xml .=
					"</contents>
		        </customs>";
	   		}
	   		/**
	   		 * END TO BE ALTERED
	   		 */

	    	$xml .= "
			<label>A4</label>
	    </shipment>";

		return ['response' => $this->apiCall($xml, 'booking'), 'xml' => $xml, 'parcels' => $parcel_list];
	}

}